@@ -4,7 +4,7 @@ methods that act upon a transfer function to gain functions such as the
 mass function.
 '''
 
-version = '1.1.2'
+version = '1.1.3'
 
 ###############################################################################
 # Some Imports
@@ -88,7 +88,9 @@ class Perturbations(object):
                             11. 'Courtin': Courtin 2011
                             12. 'Angulo': Angulo 2012
                             13. 'Angulo_Bound': Angulo sub-halo function 2012
-                            14. 'user_model': A user-input string function
+                            14. "Bhattacharya": Bhattacharya empirical fit 2011
+                            15. "Behroozi": Behroozi extension to Tinker for high-z
+                            16. 'user_model': A user-input string function
         
         delta_vir:     The virial overdensity for the halo definition
                        Default: delta_vir = 200.0
@@ -104,6 +106,9 @@ class Perturbations(object):
                        If False, will use function to calculate all values specified in M (may give ridiculous results)
                        Default: True
                        
+        R:             The distances at which the dark matter correlation function is calculated in Mpc/h
+                       Default: np.linspace(1, 200, 200)
+                       
         **kwargs:      There is a placeholder for any additional cosmological parameters, or camb
                        parameters, that one wishes to include. Parameters that aren't used won't
                        break the program, they will just be ignored. Here follows a list of parameters
@@ -158,7 +163,8 @@ class Perturbations(object):
                  transfer_file=None,
                  z=0.0,
                  wdm_mass=None, k_bounds=[0.0000001, 20000.0],
-                 delta_vir=200.0, user_fit='', transfer_fit='CAMB', cut_fit=True, ** kwargs):
+                 delta_vir=200.0, user_fit='', transfer_fit='CAMB',
+                 cut_fit=True, ** kwargs):
         """
         Initializes the cosmology for which to perform the perturbation analysis.      
         """
@@ -222,6 +228,7 @@ class Perturbations(object):
             "Crocce":self._nufnu_Crocce,
             "Courtin":self._nufnu_Courtin,
             "Bhattacharya": self._nufnu_Bhattacharya,
+            "Behroozi": self._nufnu_Tinker,
             "user_model":self._nufnu_user_model
             }
 
@@ -292,6 +299,8 @@ class Perturbations(object):
                 self.transfer_fit = kwargs['transfer_fit']
             elif key is "cut_fit":
                 self.cut_fit = kwargs['cut_fit']
+            elif key is "R":
+                self.R = kwargs['R']
 
             else:
                 print "WARNING: ", key, " is not a valid parameter for the Perturbations class"
@@ -421,9 +430,6 @@ class Perturbations(object):
 
     @k_bounds.setter
     def k_bounds(self, val):
-        if val is None:
-            self.__k_bounds = val
-            return
         try:
             if len(val) != 2:
                 raise ValueError("k_bounds must be a sequence of length 2 (lower, upper)")
@@ -435,6 +441,12 @@ class Perturbations(object):
 
         #We delete stuff directly dependent on it
         del self.lnk
+        #Wrap the following in a try: except: because self.transfer_fit may not be set yet (at instantiation)
+        try:
+            if self.transfer_fit == "EH":
+                del self._transfer_original
+        except:
+            pass
 
 
         self.__k_bounds = val
@@ -572,6 +584,8 @@ class Perturbations(object):
         del self.fsigma
         self.__cut_fit = val
 
+
+
     #--------------------------------  START NON-SET PROPERTIES ----------------------------------------------
     @property
     def dlogM(self):
@@ -895,6 +909,14 @@ class Perturbations(object):
             return self.__dndm
         except:
             self.__dndm = self.fsigma * self.cosmo_params['mean_dens'] * np.abs(self._dlnsdlnm) / self.M ** 2
+            if self.mf_fit == 'Behroozi':
+                a = 1 / (1 + self.z)
+                theta = 0.144 / (1 + np.exp(14.79 * (a - 0.213))) * (self.M / 10 ** 11.5) ** (0.5 / (1 + np.exp(6.5 * a)))
+                ngtm_tinker = self._ngtm()
+                ngtm_behroozi = 10 ** (theta + np.log10(ngtm_tinker))
+                dthetadM = 0.144 / (1 + np.exp(14.79 * (a - 0.213))) * (0.5 / (1 + np.exp(6.5 * a))) * (self.M / 10 ** 11.5) ** (0.5 / (1 + np.exp(6.5 * a)) - 1) / (10 ** 11.5)
+                self.__dndm = self.__dndm * 10 ** theta - ngtm_behroozi * np.log(10) * dthetadM
+
             return self.__dndm
 
     @dndm.deleter
@@ -948,6 +970,31 @@ class Perturbations(object):
         except:
             pass
 
+    def _ngtm(self):
+        ngtm = np.zeros_like(self.dndlnm)
+
+        # set M and mass_function within computed range
+        M = self.M[np.logical_not(np.isnan(self.dndlnm))]
+        mass_function = np.log(self.dndlnm[np.logical_not(np.isnan(self.dndlnm))])
+
+        # Interpolate the mass_function - this is in log-log space.
+        mf = spline(np.log(M), mass_function, k=1)
+
+        # Define max_M as either 18 or the maximum set by user
+        max_M = np.log(np.max([10 ** 18, M[-1]]))
+
+        for i, m in enumerate(self.M):
+            if np.isnan(m):
+                ngtm[i] = np.nan
+            else:
+                # Set up new grid with 4097 steps from m to M=17
+                M_new, dlnM = np.linspace(np.log(m), max_M, 4097, retstep=True)
+                mf_new = mf(M_new)
+
+                ngtm[i] = intg.romb(np.exp(mf_new), dx=dlnM)
+
+        return ngtm
+
     @property
     def ngtm(self):
         """
@@ -957,27 +1004,33 @@ class Perturbations(object):
             return self.__ngtm
         except:
             # Initialize the function
-            self.__ngtm = np.zeros_like(self.dndlnm)
-
-            # set M and mass_function within computed range
-            M = self.M[np.logical_not(np.isnan(self.dndlnm))]
-            mass_function = np.log(self.dndlnm[np.logical_not(np.isnan(self.dndlnm))])
-
-            # Interpolate the mass_function - this is in log-log space.
-            mf = spline(np.log(M), mass_function, k=1)
-
-            # Define max_M as either 18 or the maximum set by user
-            max_M = np.log(np.max([10 ** 18, M[-1]]))
-
-            for i, m in enumerate(self.M):
-                if np.isnan(m):
-                    self.__ngtm[i] = np.nan
-                else:
-                    # Set up new grid with 4097 steps from m to M=17
-                    M_new, dlnM = np.linspace(np.log(m), max_M, 4097, retstep=True)
-                    mf_new = mf(M_new)
-
-                    self.__ngtm[i] = intg.romb(np.exp(mf_new), dx=dlnM)
+#            self.__ngtm = np.zeros_like(self.dndlnm)
+#
+#            # set M and mass_function within computed range
+#            M = self.M[np.logical_not(np.isnan(self.dndlnm))]
+#            mass_function = np.log(self.dndlnm[np.logical_not(np.isnan(self.dndlnm))])
+#
+#            # Interpolate the mass_function - this is in log-log space.
+#            mf = spline(np.log(M), mass_function, k=1)
+#
+#            # Define max_M as either 18 or the maximum set by user
+#            max_M = np.log(np.max([10 ** 18, M[-1]]))
+#
+#            for i, m in enumerate(self.M):
+#                if np.isnan(m):
+#                    self.__ngtm[i] = np.nan
+#                else:
+#                    # Set up new grid with 4097 steps from m to M=17
+#                    M_new, dlnM = np.linspace(np.log(m), max_M, 4097, retstep=True)
+#                    mf_new = mf(M_new)
+#
+#                    self.__ngtm[i] = intg.romb(np.exp(mf_new), dx=dlnM)
+#
+#            #Here we add a correction for high-z for Tinker, by Behroozi.
+#            if self.mf_fit == 'Behroozi':
+#                pivot_factor = 0.144 / (1 + np.exp(14.79 * (1 / (1 + self.z)) - 0.213))
+#                self.__ngtm = 10 ** (pivot_factor * (self.M / (10 ** 11.5)) ** (0.5 / (1 + np.exp(6.5 * (1 / (1 + self.z))))) + np.log10(self.__ngtm))
+            self.__ngtm = self._ngtm()
             return self.__ngtm
 
     @ngtm.deleter
@@ -1361,6 +1414,7 @@ class Perturbations(object):
             vfv[np.logical_or(self.M < 6 * 10 ** 11, self.M > 3 * 10 ** 15)] = np.NaN
 
         return vfv
+
     def _nufnu_user_model(self):
         """
         Calculates vfv based on a user-input model.
@@ -1374,8 +1428,6 @@ class Perturbations(object):
 
 
 
-
-
 if __name__ == "__main__":
     M = np.arange(10, 15, 0.01)
     pert = Perturbations(M)
